@@ -43,32 +43,34 @@ from .data_session import H5SessionData
 class DataManager(DbManager):
     """ Main class that will manage the sessions and their information.
     """
-    def __init__(self, dataPath, dbName='emhub.sqlite', user=None, cleanDb=False):
+    def __init__(self, dataPath, dbName='emhub.sqlite',
+                 user=None, cleanDb=False, create=True):
         self._dataPath = dataPath
         self._sessionsPath = os.path.join(dataPath, 'sessions')
 
         # Initialize main database
         dbPath = os.path.join(dataPath, dbName)
-        self.init_db(dbPath, cleanDb=cleanDb)
-
-        # Create a separate database for logs
-        logDbPath = dbPath.replace('.sqlite', '-logs.sqlite')
-        self._db_log = DataLog(logDbPath, cleanDb=cleanDb)
+        self.init_db(dbPath, cleanDb=cleanDb, create=create)
 
         self._lastSession = None
         self._user = user  # Logged user
 
-        # Create sessions dir if not exists
-        os.makedirs(self._sessionsPath, exist_ok=True)
+        if create:
+            # Create a separate database for logs
+            logDbPath = dbPath.replace('.sqlite', '-logs.sqlite')
+            self._db_log = DataLog(logDbPath, cleanDb=cleanDb)
+
+            # Create sessions dir if not exists
+            os.makedirs(self._sessionsPath, exist_ok=True)
 
     def _create_models(self):
         """ Function called from the init_db method. """
         create_data_models(self)
 
     def log(self, log_type, log_name, *args, **kwargs):
-        user_id = None if self._user is None else self._user.id
+        log_user_id = None if self._user is None else self._user.id
 
-        self._db_log.log(user_id, log_type, log_name,
+        self._db_log.log(log_user_id, log_type, log_name,
                          *args, **kwargs)
 
     def get_logs(self):
@@ -349,19 +351,11 @@ class DataManager(DbManager):
         data = H5SessionData(self._session_data_path(session), mode='a')
         data.close()
 
-        self.log("operation", "create_Session",
-                 attrs=self.json_from_object(session))
-
         return session
 
     def update_session(self, **attrs):
         """ Update session attrs. """
-        result = self.__update_item(self.Session, **attrs)
-
-        self.log("operation", "update_Session",
-                 attrs=self.json_from_dict(attrs))
-
-        return result
+        return self.__update_item(self.Session, **attrs)
 
     def delete_session(self, **attrs):
         """ Remove a session row. """
@@ -386,6 +380,72 @@ class DataManager(DbManager):
         session.data = H5SessionData(self._session_data_path(session), mode)
         return session
 
+    # -------------------------- INVOICE PERIODS ------------------------------
+    def get_invoice_periods(self, condition=None, orderBy=None, asJson=False):
+        """ Returns a list.
+        condition example: text("id<:value and name=:name")
+        """
+        return self.__items_from_query(self.InvoicePeriod,
+                                       condition=condition,
+                                       orderBy=orderBy,
+                                       asJson=asJson)
+
+    def create_invoice_period(self, **attrs):
+        """ Add a new session row. """
+        return self.__create_item(self.InvoicePeriod, **attrs)
+
+    def update_invoice_period(self, **attrs):
+        """ Update session attrs. """
+        return self.__update_item(self.InvoicePeriod, **attrs)
+
+    def delete_invoice_period(self, **attrs):
+        """ Remove a session row. """
+        periodId = attrs['id']
+        period = self.InvoicePeriod.query.get(periodId)
+        self.delete(period)
+
+        self.log("operation", "delete_InvoicePeriod",
+                 attrs=self.json_from_dict(attrs))
+
+        return period
+
+    def get_invoice_period_by(self, **kwargs):
+        """ This should return a single user or None. """
+        return self.__item_by(self.InvoicePeriod, **kwargs)
+
+    # ---------------------------- TRANSACTIONS -------------------------------
+    def get_transactions(self, condition=None, orderBy=None, asJson=False):
+        """ Returns a list.
+        condition example: text("id<:value and name=:name")
+        """
+        return self.__items_from_query(self.Transaction,
+                                       condition=condition,
+                                       orderBy=orderBy,
+                                       asJson=asJson)
+
+    def create_transaction(self, **attrs):
+        """ Add a new session row. """
+        return self.__create_item(self.Transaction, **attrs)
+
+    def update_transaction(self, **attrs):
+        """ Update session attrs. """
+        return self.__update_item(self.Transaction, **attrs)
+
+    def delete_transaction(self, **attrs):
+        """ Remove a session row. """
+        transactionId = attrs['id']
+        transaction = self.Transaction.query.get(transactionId)
+        self.delete(transaction)
+
+        self.log("operation", "delete_Transaction",
+                 attrs=self.json_from_dict(attrs))
+
+        return transaction
+
+    def get_transaction_by(self, **kwargs):
+        """ This should return a single user or None. """
+        return self.__item_by(self.Transaction, **kwargs)
+
     # --------------- Internal implementation methods -------------------------
     def __create_item(self, ModelClass, **attrs):
         new_item = ModelClass(**attrs)
